/// <reference types="node" />
import { Draft } from 'immer';
import { DebounceOptions } from './helpers/debounce';
import { Cancel } from './helpers/misc';
declare type Result<Value> = {
    kind: 'value';
    value: Value;
    t: Date;
} | {
    kind: 'error';
    error: unknown;
    t: Date;
};
declare type Instance<Arg, Value> = {
    arg: Arg;
    current?: Result<Value>;
    inProgress?: Promise<Value>;
    invalid?: boolean;
    invalidateTimer?: NodeJS.Timeout;
    clearTimer?: NodeJS.Timeout;
};
export declare class Action<Arg, Value> {
    private action;
    private options;
    static allActions: Action<any, any>[];
    static options: {
        invalidateAfter?: number | ((value: unknown | undefined, error: unknown | undefined) => number | undefined);
        clearAfter?: number | ((value: unknown | undefined, error: unknown | undefined) => number | undefined);
    };
    static clearCacheAll(): void;
    private cache;
    constructor(action: (arg: Arg) => Promise<Value>, options?: {
        invalidateAfter?: number | ((value: Value | undefined, error: unknown | undefined) => number | undefined);
        clearAfter?: number | ((value: Value | undefined, error: unknown | undefined) => number | undefined);
    });
    getCache(arg: Arg): Instance<Arg, Value> | undefined;
    getCacheValue(arg: Arg): Value | undefined;
    getCacheError(arg: Arg): unknown;
    setCache(arg: Arg, value: Value): void;
    updateCache(arg: Arg, update: (draft: Draft<Value>) => void): boolean;
    updateCacheAll(update: (draft: Draft<Value>, arg: Arg) => void): void;
    clearCache(arg: Arg): void;
    clearCacheAll(): void;
    invalidateCache(arg: Arg): void;
    invalidateCacheAll(): void;
    get(arg: Arg, { clearBeforeUpdate, retries }?: {
        clearBeforeUpdate?: boolean | undefined;
        retries?: number | undefined;
    }): Promise<Value>;
    execute(arg: Arg, { clearBeforeUpdate, retries }?: {
        clearBeforeUpdate?: boolean | undefined;
        retries?: number | undefined;
    }): Promise<Value>;
    setTimers(arg: Arg): void;
    subscribe(arg: Arg, listener: (value: Value | undefined, state: {
        error?: unknown;
        isLoading: boolean;
    }, instance: Instance<Arg, Value>) => void, { runNow, throttle, debounce }?: {
        runNow?: boolean;
        throttle?: number;
        debounce?: number | DebounceOptions;
    }): Cancel;
    private updateInstance;
}
export {};
